@@ -4,6 +4,6 @@ from __future__ import annotations
 
 
 class FilesWatcher:  # noqa: D101
-    def __init__(self) -> None:
+    def __init__(self) -> None:  # pragma: no cover
         self.prev_included_files: list[str] = []
         self.included_files: list[str] = []
